print("import: 'virtualenv'")
import virtualenv

print("import: 'virtualenv.activation'")
import virtualenv.activation

print("import: 'virtualenv.app_data'")
import virtualenv.app_data

print("import: 'virtualenv.config'")
import virtualenv.config

print("import: 'virtualenv.create'")
import virtualenv.create

print("import: 'virtualenv.discovery'")
import virtualenv.discovery

print("import: 'virtualenv.run'")
import virtualenv.run

print("import: 'virtualenv.seed'")
import virtualenv.seed

print("import: 'virtualenv.util'")
import virtualenv.util

